@@ -1,16 +1,24 @@
 # -*- coding: utf-8 -*-
+from __future__ import division
 import glob
+import sys
 import igo.util as util
 from igo.util import FileMappedInputStream
 from igo.trie import Searcher
 
 
+if sys.version_info[0] > 2:
+    space = ' '
+else:
+    space = unichr(0x20)
+
+
 class ViterbiNode(object):
     """
     Viterbiアルゴリズムで使用されるノード
     """
-    def __init__(self, wordId, start, length, leftId, rightId, isSpace):
-        self.cost = 0
+    def __init__(self, wordId, start, length, cost, leftId, rightId, isSpace):
+        self.cost = cost
         """ 始点からノードまでの総コスト """
         self.prev = None
         """ コスト最小の前方のノードへのリンク """
@@ -29,7 +37,7 @@ class ViterbiNode(object):
 
     @staticmethod
     def makeBOSEOS():
-        return ViterbiNode(0, 0, 0, 0, 0, False)
+        return ViterbiNode(0, 0, 0, 0, 0, 0, False)
 
 
 class CharCategory:
@@ -37,8 +45,8 @@ class CharCategory:
         self.categorys = CharCategory.readCategorys(dataDir, bigendian)
         fmis = FileMappedInputStream(dataDir + "/code2category", bigendian)
         try:
-            self.char2id = fmis.getIntArray(fmis.size() / 4 / 2)
-            self.eqlMasks = fmis.getIntArray(fmis.size() / 4 / 2)
+            self.char2id = fmis.getIntArray(fmis.size() // 4 // 2)
+            self.eqlMasks = fmis.getIntArray(fmis.size() // 4 // 2)
         finally:
             fmis.close()
 
@@ -51,11 +59,11 @@ class CharCategory:
     @staticmethod
     def readCategorys(dataDir, bigendian):
         data = util.getIntArray(dataDir + "/char.category", bigendian)
-        size = len(data) / 4
+        size = len(data) // 4
         ary = []
         for i in range(0, size):
-            ary.append(
-                Category(data[i * 4], data[i * 4 + 1], data[i * 4 + 2] == 1, data[i * 4 + 3] == 1))
+            ary.append(Category(data[i * 4], data[i * 4 + 1],
+                                data[i * 4 + 2] == 1, data[i * 4 + 3] == 1))
         return ary
 
 
@@ -84,7 +92,7 @@ class Matrix:
         """
         形態素同士の連接コストを求める
         """
-        return self.matrix[rightId * self.rightSize + leftId]
+        return self.matrix[rightId * self.leftSize + leftId]
 
 
 class Unknown:
@@ -94,30 +102,35 @@ class Unknown:
     def __init__(self, dataDir, bigendian=False):
         self.category = CharCategory(dataDir, bigendian)
         """文字カテゴリ管理クラス"""
-        self.spaceId = self.category.category(u' ').id  # NOTE: ' 'の文字カテゴリはSPACEに予約されている
+        # NOTE: ' 'の文字カテゴリはSPACEに予約されている
+        self.spaceId = self.category.category(space).id
         """文字カテゴリがSPACEの文字のID"""
 
-    def search(self, text, start, wdic, result):
+    def search(self, text, start, wdic, callback):
+        category = self.category
         ch = text[start]
-        ct = self.category.category(ch)
+        ct = category.category(ch)
         length = len(text)
 
-        if result and not ct.invoke:
+        if not callback.isEmpty() and not ct.invoke:
             return
 
         isSpace = ct.id == self.spaceId
         limit = min(length, ct.length + start)
         for i in range(start, limit):
-            wdic.searchFromTrieId(ct.id, start, (i - start) + 1, isSpace, result)
-            if i + 1 != limit and not self.category.isCompatible(ch, text[i + 1]):
+            wdic.searchFromTrieId(ct.id, start,
+                                  (i - start) + 1, isSpace, callback)
+            if i + 1 != limit and not category.isCompatible(ch, text[i + 1]):
                 return
 
         if ct.group and limit < length:
             for i in range(limit, length):
-                if not self.category.isCompatible(ch, text[i]):
-                    wdic.searchFromTrieId(ct.id, start, i - start, isSpace, result)
+                if not category.isCompatible(ch, text[i]):
+                    wdic.searchFromTrieId(ct.id, start,
+                                          i - start, isSpace, callback)
                     return
-            wdic.searchFromTrieId(ct.id, start, length - start, isSpace, result)
+            wdic.searchFromTrieId(ct.id, start,
+                                  length - start, isSpace, callback)
 
 
 class WordDic:
@@ -132,7 +145,7 @@ class WordDic:
 
         fmis = FileMappedInputStream(dataDir + "/word.inf", bigendian)
         try:
-            wordCount = fmis.size() / (4 + 2 + 2 + 2)
+            wordCount = fmis.size() // (4 + 2 + 2 + 2)
             self.dataOffsets = fmis.getIntArray(wordCount)
             """ dataOffsets[単語ID] = 単語の素性データの開始位置 """
             self.leftIds = fmis.getShortArray(wordCount)
@@ -144,25 +157,28 @@ class WordDic:
         finally:
             fmis.close()
 
-    def cost(self, wordId):
-        return self.costs[wordId]
-
-    def search(self, text, start, result):
-        indices = self.indices
+    def search(self, text, start, callback):
+        costs = self.costs
         leftIds = self.leftIds
         rightIds = self.rightIds
+        indices = self.indices
 
-        def collect(start, offset, trieId):
+        def fn(start, offset, trieId):
             end = indices[trieId + 1]
             for i in range(indices[trieId], end):
-                result.append(ViterbiNode(i, start, offset, self.leftIds[i], self.rightIds[i], False))
+                callback(ViterbiNode(i, start, offset, costs[i],
+                                     leftIds[i], rightIds[i], False))
 
-        self.trie.eachCommonPrefix(text, start, collect)
+        self.trie.eachCommonPrefix(text, start, fn)
 
-    def searchFromTrieId(self, trieId, start, wordLength, isSpace, result):
+    def searchFromTrieId(self, trieId, start, wordLength, isSpace, callback):
+        costs = self.costs
+        leftIds = self.leftIds
+        rightIds = self.rightIds
         end = self.indices[trieId + 1]
         for i in range(self.indices[trieId], end):
-            result.append(ViterbiNode(i, start, wordLength, self.leftIds[i], self.rightIds[i], isSpace))
+            callback(ViterbiNode(i, start, wordLength, costs[i],
+                                 leftIds[i], rightIds[i], isSpace))
 
     def wordData(self, wordId):
         return self.data[self.dataOffsets[wordId]:self.dataOffsets[wordId + 1]]
